@@ -29,7 +29,7 @@ import com.google.common.base.Splitter;
 import com.google.common.collect.Sets;
 import org.apache.commons.lang3.StringUtils;
 
-import org.apache.cassandra.config.Schema;
+import org.apache.cassandra.schema.Schema;
 import org.apache.cassandra.cql3.CQL3Type;
 import org.apache.cassandra.cql3.functions.Function;
 import org.apache.cassandra.cql3.functions.FunctionName;
@@ -135,6 +135,11 @@ public class FunctionResource implements IResource
         return new FunctionResource(keyspace, name, argTypes);
     }
 
+    public static FunctionResource function(Function function)
+    {
+        return new FunctionResource(function.name().keyspace, function.name().name, function.argTypes());
+    }
+
     /**
      * Creates a FunctionResource representing a specific, keyspace-scoped function.
      * This variant is used to create an instance during parsing of a CQL statement.
@@ -151,32 +156,29 @@ public class FunctionResource implements IResource
         if (keyspace == null)
             throw new InvalidRequestException("In this context function name must be " +
                                               "explictly qualified by a keyspace");
-        List<AbstractType<?>> abstractTypes = new ArrayList<>();
+        List<AbstractType<?>> abstractTypes = new ArrayList<>(argTypes.size());
         for (CQL3Type.Raw cqlType : argTypes)
             abstractTypes.add(cqlType.prepare(keyspace).getType());
 
         return new FunctionResource(keyspace, name, abstractTypes);
     }
 
+    public static FunctionResource functionFromCql(FunctionName name, List<CQL3Type.Raw> argTypes)
+    {
+        return functionFromCql(name.keyspace, name.name, argTypes);
+    }
+
     /**
      * Parses a resource name into a FunctionResource instance.
-     * A valid resource name for function should be in the follow format:
-     * functions/KEYSPACE/FUNCTION_NAME[FUNCTION_ARGS]
-     * Note that
-     * 1. FUNCTION_NAME could contain any character due to the use of quoted text in CQL.
-     * 2. FUNCTION_ARGS could be empty. If it is not empty, it is expressed in this format:
-     *    FUNCTION_ARG1^FUNCTION_ARG2... where ^ is the delimiter for arguments
      *
      * @param name Name of the function resource.
      * @return FunctionResource instance matching the name.
      */
     public static FunctionResource fromName(String name)
     {
-        // Split the name into at most 3 parts.
-        // The last part is the function name + args list, the name might contains '/'
-        String[] parts = StringUtils.split(name, "/", 3);
+        String[] parts = StringUtils.split(name, '/');
 
-        if (!parts[0].equals(ROOT_NAME))
+        if (!parts[0].equals(ROOT_NAME) || parts.length > 3)
             throw new IllegalArgumentException(String.format("%s is not a valid function resource name", name));
 
         if (parts.length == 1)
@@ -187,17 +189,8 @@ public class FunctionResource implements IResource
 
         if (!name.matches("^.+\\[.*\\]$"))
             throw new IllegalArgumentException(String.format("%s is not a valid function resource name. It must end with \"[]\"", name));
-
-        String function = parts[2];
-        // The name must end with '[...]' block
-        int lastStartingBracketIndex = function.lastIndexOf('[');
-        String functionName = StringUtils.substring(function, 0, lastStartingBracketIndex);
-        String functionArgs = StringUtils.substring(function,
-                                                    // excludes the wrapping brackets [ ]
-                                                    lastStartingBracketIndex + 1,
-                                                    function.length() - 1);
-
-        return function(parts[1], functionName, functionArgs.isEmpty() ? Collections.emptyList() : argsListFromString(functionArgs));
+        String[] nameAndArgs = StringUtils.split(parts[2], "[|]");
+        return function(parts[1], nameAndArgs[0], nameAndArgs.length > 1 ? argsListFromString(nameAndArgs[1]) : Collections.emptyList());
     }
 
     /**
