@@ -787,27 +787,29 @@ public class Instance extends IsolatedExecutor implements IInvokableInstance
                                 () -> Ref.shutdownReferenceReaper(1L, MINUTES),
                                 () -> Memtable.MEMORY_POOL.shutdownAndWait(1L, MINUTES),
                                 () -> DiagnosticSnapshotService.instance.shutdownAndWait(1L, MINUTES),
-                                () -> ScheduledExecutors.shutdownAndWait(1L, MINUTES),
                                 () -> SSTableReader.shutdownBlocking(1L, MINUTES),
                                 () -> shutdownAndWait(Collections.singletonList(ActiveRepairService.repairCommandExecutor()))
             );
 
-            error = parallelRun(error, executor, () -> ScheduledExecutors.shutdownAndWait(1L, MINUTES));
-
             internodeMessagingStarted = false;
             error = parallelRun(error, executor,
-                                CommitLog.instance::shutdownBlocking,
                                 // can only shutdown message once, so if the test shutsdown an instance, then ignore the failure
                                 (IgnoreThrowingRunnable) () -> MessagingService.instance().shutdown(1L, MINUTES, false, true)
             );
+
             error = parallelRun(error, executor,
-                                () -> GlobalEventExecutor.INSTANCE.awaitInactivity(1l, MINUTES),
+                                () -> GlobalEventExecutor.INSTANCE.awaitInactivity(1L, MINUTES),
                                 () -> Stage.shutdownAndWait(1L, MINUTES),
                                 () -> SharedExecutorPool.SHARED.shutdownAndWait(1L, MINUTES)
             );
-            error = parallelRun(error, executor,
-                                () -> shutdownAndWait(Collections.singletonList(JMXBroadcastExecutor.executor))
-            );
+
+            // CommitLog must shut down after Stage, or threads from the latter may attempt to use the former.
+            // (ex. A Mutation stage thread may attempt to add a mutation to the CommitLog.)
+            error = parallelRun(error, executor, CommitLog.instance::shutdownBlocking);
+            error = parallelRun(error, executor, () -> shutdownAndWait(Collections.singletonList(JMXBroadcastExecutor.executor)));
+            
+            // ScheduledExecutors shuts down after MessagingService, as MessagingService may issue tasks to it.
+            error = parallelRun(error, executor, () -> ScheduledExecutors.shutdownAndWait(1L, MINUTES));
 
             Throwables.maybeFail(error);
         }).apply(isolatedExecutor);
