@@ -19,22 +19,18 @@
 package org.apache.cassandra.tools;
 
 import org.junit.Test;
-import org.junit.runner.RunWith;
 
 import com.datastax.driver.core.exceptions.NoHostAvailableException;
-import org.apache.cassandra.OrderedJUnit4ClassRunner;
 import org.apache.cassandra.tools.ToolRunner.ToolResult;
 import org.hamcrest.CoreMatchers;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertThat;
-import static org.junit.Assert.fail;
 
-@RunWith(OrderedJUnit4ClassRunner.class)
 public class BulkLoaderTest extends OfflineToolUtils
 {
     @Test
-    public void testBulkLoader_NoArgs() throws Exception
+    public void testBulkLoader_NoArgs()
     {
         ToolResult tool = ToolRunner.invokeClass(BulkLoader.class);
         assertEquals(1, tool.getExitCode());
@@ -59,7 +55,7 @@ public class BulkLoaderTest extends OfflineToolUtils
         assertEquals(-1, tool.getExitCode());
         if (!(tool.getException().getCause() instanceof BulkLoadException))
             throw tool.getException();
-        if (!(tool.getException().getCause().getCause() instanceof NoHostAvailableException))
+        if (!(tool.getException().getCause().getCause().getCause() instanceof NoHostAvailableException))
             throw tool.getException();
 
         assertNoUnexpectedThreadsStarted(null, new String[]{"globalEventExecutor-1-1", "globalEventExecutor-1-2"});
@@ -83,7 +79,7 @@ public class BulkLoaderTest extends OfflineToolUtils
         assertEquals(-1, tool.getExitCode());
         if (!(tool.getException().getCause() instanceof BulkLoadException))
             throw tool.getException();
-        if (!(tool.getException().getCause().getCause() instanceof NoHostAvailableException))
+        if (!(tool.getException().getCause().getCause().getCause() instanceof NoHostAvailableException))
             throw tool.getException();
 
         assertNoUnexpectedThreadsStarted(null, new String[] { "globalEventExecutor-1-1", "globalEventExecutor-1-2" });
@@ -107,7 +103,7 @@ public class BulkLoaderTest extends OfflineToolUtils
         assertEquals(-1, tool.getExitCode());
         if (!(tool.getException().getCause() instanceof BulkLoadException))
             throw tool.getException();
-        if (!(tool.getException().getCause().getCause() instanceof NoHostAvailableException))
+        if (!(tool.getException().getCause().getCause().getCause() instanceof NoHostAvailableException))
             throw tool.getException();
 
         assertNoUnexpectedThreadsStarted(null, new String[] { "globalEventExecutor-1-1", "globalEventExecutor-1-2" });
@@ -128,7 +124,7 @@ public class BulkLoaderTest extends OfflineToolUtils
                                                  "9041",
                                                  OfflineToolUtils.sstableDirName("legacy_sstables", "legacy_ma_simple"));
         assertEquals(-1, tool.getExitCode());
-        throw tool.getException().getCause().getCause();
+        throw tool.getException().getCause().getCause().getCause();
     }
 
     @Test(expected = NoHostAvailableException.class)
@@ -139,6 +135,6 @@ public class BulkLoaderTest extends OfflineToolUtils
                                                  "127.9.9.1:9041",
                                                  OfflineToolUtils.sstableDirName("legacy_sstables", "legacy_ma_simple"));
         assertEquals(-1, tool.getExitCode());
-        throw tool.getException().getCause().getCause();
+        throw tool.getException().getCause().getCause().getCause();
     }
 }
